# $Author: jeffr $
# $Date: 2022-11-11 10:33:14 -0500 (Fri, 11 Nov 2022) $
# $Revision: 30 $

#
# Decompress gzipped files
#

Function Decompress-File {
    Param (
        $infile,
        $outfile = ($infile -replace '\.gz$','')
    )

    $inp = New-Object System.IO.FileStream $inFile, ([IO.FileMode]::Open), ([IO.FileAccess]::Read), ([IO.FileShare]::Read)
    $out = New-Object System.IO.FileStream $outFile, ([IO.FileMode]::Create), ([IO.FileAccess]::Write), ([IO.FileShare]::None)
    $gzipStream = New-Object System.IO.Compression.GzipStream $inp, ([IO.Compression.CompressionMode]::Decompress)
    $buffer = New-Object byte[](1024)

    while ($true) {
        $read = $gzipstream.Read($buffer, 0, 1024)
        if ($read -le 0){break}
        $out.Write($buffer, 0, $read)
    }
    
    $gzipStream.Close()
    $out.Close()
    $inp.Close()
}

#
# Get config variables
#

If ($args.Length -lt 2) {
    Write-Host 'Usage .\radar.ps1 {radar ID} {animation duration in mins} {delete files in archive}'
    Exit 1
}

$radar_ids = 'ABC', 'ABR', 'ABX', 'ACG', 'AEC', 'AHG', 'AIH', 'AKC', 'AKQ', 'AMA', 'AMX', 'APD', 'APX', 'ARX', 'ATX', 'BBX', 'BGM', 'BHX', 'BIS', 'BLX', 'BMX', 'BOX', 'BRO', 'BUF', 'BYX', 'CAE', 'CBW', 'CBX', 'CCX', 'CLE', 'CLX', 'CRP', 'CXX', 'CYS', 'DAX', 'DDC', 'DFX', 'DGX', 'DIX', 'DLH', 'DMX', 'DOX', 'DTX', 'DVN', 'DYX', 'EAX', 'EMX', 'ENX', 'EOX', 'EPZ', 'ESX', 'EVX', 'EWX', 'EYX', 'FCX', 'FDR', 'FDX', 'FFC', 'FSD', 'FSX', 'FTG', 'FWS', 'GGW', 'GJX', 'GLD', 'GRB', 'GRK', 'GRR', 'GSP', 'GUA', 'GWX', 'GYX', 'HDX', 'HGX', 'HKI', 'HKM', 'HMO', 'HNX', 'HPX', 'HTX', 'HWA', 'ICT', 'ICX', 'ILN', 'ILX', 'IND', 'INX', 'IWA', 'IWX', 'JAX', 'JGX', 'JKL', 'JUA', 'LBB', 'LCH', 'LGX', 'LIX', 'LNX', 'LOT', 'LRX', 'LSX', 'LTX', 'LVX', 'LWX', 'LZK', 'MAF', 'MAX', 'MBX', 'MHX', 'MKX', 'MLB', 'MOB', 'MPX', 'MQT', 'MRX', 'MSX', 'MTX', 'MUX', 'MVX', 'MXX', 'NKX', 'NQA', 'OAX', 'OHX', 'OKX', 'OTX', 'PAH', 'PBZ', 'PDT', 'POE', 'PUX', 'RAX', 'RGX', 'RIW', 'RLX', 'RTX', 'SFX', 'SGF', 'SHV', 'SJT', 'SOX', 'SRX', 'TBW', 'TFX', 'TLH', 'TLX', 'TWX', 'TYX', 'UDX', 'UEX', 'VAX', 'VBX', 'VNX', 'VTX', 'VWX', 'YUX'

# Was a valid radar ID provided?
If ($radar_ids.IndexOf($args[0]) -lt 0) {
    Write-Host 'Radar ID'  $args[0] 'not found. Valid radar IDs include:'
    Write-Host $radar_ids
    Write-Host 'Refer to https://www.weather.gov/jetstream/ridge_download'
    Write-Host 'Quitting.'
    Exit 1
}

#
# Basic variables for processing
#

$image_freq = 4               # How often radar images are updated by the NWS
$resize_amt ='25%'            # Amount to shrink the images by
$animate_dur = $args[1]       # Duration (in minutes) to depict in animation
$radar_bg = 'background.gif'  # Background image (e.g. map) to use
$fill_color = 'white'         # Font color to use for radar image info

# Number of files to download
$num_files = $animate_dur / $image_freq
# Radar images reside at this URL
$img_url = 'https://mrms.ncep.noaa.gov/data/RIDGEII/L2/K' + $args[0] + '/BREF_RAW/'

#
# Get the list of radar image files and download files
#

# Get the entire list of files
$list_content = Invoke-WebRequest -UseBasicParsing -URI $img_url
$list_content.Links | ForEach-Object {$_.href} {
    Add-Content .\tmp\radar.txt $_.href
}

# Get the list of files required
$files = (Get-Content .\tmp\radar.txt -Tail $num_files)

# Download the files if necessary
foreach ($file in $files) {
    if (-not (Test-Path .\archive\$file -PathType Leaf)) {
        $img_url_full = $img_url + $file
        $out_file = 'archive\' + $file
        Invoke-WebRequest -UseBasicParsing -URI $img_url_full -OutFile $out_file
    }
}

#
# Process image files
#

foreach ($file in $files) {
    # Decompress the file
    $inp_img = (Get-Location).path + '\archive\' + $file
    $out_img = ($inp_img -replace '\.gz$','')
    $out_tmp = ($out_img -replace 'archive','tmp')
    Decompress-File $inp_img $out_tmp

    # Resize and convert
    magick.exe convert -quiet $out_tmp -resize $resize_amt "$out_tmp.gif"
    # Combine with background
    magick.exe convert -quiet $radar_bg "$out_tmp.gif" -composite +dither -colors 128 "$out_tmp.gif"
    # Add a label
    mogrify.exe -quiet -gravity SouthEast -pointsize 20 -fill $fill_color -annotate +0+100 ($file -replace '\.tif.gz$','') "$out_tmp.gif"
    # Delete original
    Remove-Item $out_tmp
}

# Copy last (latest) file and create animation
Copy-Item "$out_tmp.gif" .\radar.gif
magick.exe convert -quiet -delay 20 -loop 0 '.\tmp\*.gif' +dither -fuzz 25% -layers optimize-plus animation.gif

#
# Clean up
#

# Delete files in tmp dir
Remove-Item .\tmp\*

# Delete radar images in archive
If ($args[2] -eq 'delete') {
    Remove-Item .\archive\*
}

Exit 0